//---------------------------------------------------------------------------------------------------------------------------------------------------
// <copyright file="CutScene.cs" company="DarkWynter Studios">
//     Copyright (C)2007 DarkWynter Studios.  All rights reserved.
// </copyright>
//---------------------------------------------------------------------------------------------------------------------------------------------------
// {Contact : darkwynter.com for licensing information
//---------------------------------------------------------------------------------------------------------------------------------------------------

namespace ElementalGame.GraphicUserInterface
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using Microsoft.Xna.Framework;
    using Microsoft.Xna.Framework.Audio;
    using Microsoft.Xna.Framework.Content;
    using Microsoft.Xna.Framework.Graphics;
    using Microsoft.Xna.Framework.Input;
    using Microsoft.Xna.Framework.Storage;
    #endregion
    
    public class Line
    {
        List<Actor> actors;         // Actors in the Scene
        Background background;      // The Scene
        int Timer;                  // Length of time to render this line             
    }

    public class Actor
    {
        Texture2D image;            // Background Picture
        string line;                // Text writer
        string audio;               // Audio Track
    }
    public class Background
    {
        Texture2D image;            // Background Picture
        string audio;               // Audio Track
    }

    // The general CutScene... Screen
    public class CutScene : GameScreen
    {
        protected List<Texture2D> screens = new List<Texture2D>();
        protected List<int> screenTimeDelays = new List<int>();
        List<Line> lines = new List<Line>();

        protected int currentScreenIndex = 0;

        protected System.Diagnostics.Stopwatch stopWatch = new System.Diagnostics.Stopwatch();
        protected int currentTimeDelay = 0;
        protected bool isFinished = false;

        // Constructor:
        public CutScene()
        {
            //Instantiate the GameMenu
            menu = new GameMenu("", 0, 0);
            //XML.LoadScene("CutScenes/Scene1.xml");
        }

        public void StartCutScene()
        {
            stopWatch.Reset();
            stopWatch.Start();
            currentScreenIndex = 0;
            SetBackground(screens[currentScreenIndex]);
            currentTimeDelay = screenTimeDelays[currentScreenIndex];

            isFinished = false;
        }

        public bool IsDone()
        {
            return isFinished;
        }

        public void Skip()
        {
            isFinished = true;
        }

        public new void Draw(SpriteBatch spriteBatch)
        {
            if (stopWatch.ElapsedMilliseconds >= currentTimeDelay)
            {
                if (currentScreenIndex == screens.Count - 1)
                {
                    isFinished = true;
                }
                else
                {
                    currentScreenIndex++;
                    SetBackground(screens[currentScreenIndex]);
                    currentTimeDelay = screenTimeDelays[currentScreenIndex];
                }
                stopWatch.Reset();
                stopWatch.Start();
            }

            base.Draw(spriteBatch);
        }
    }
}