//---------------------------------------------------------------------------------------------------------------------------------------------------
// <copyright file="MenuSystem.cs" company="DarkWynter Studios">
//     Copyright (C)2007 DarkWynter Studios.  All rights reserved.
// </copyright>
//---------------------------------------------------------------------------------------------------------------------------------------------------
// {Contact : darkwynter.com for licensing information
//---------------------------------------------------------------------------------------------------------------------------------------------------

namespace ElementalGame.GraphicUserInterface
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using Microsoft.Xna.Framework;
    using Microsoft.Xna.Framework.Audio;
    using Microsoft.Xna.Framework.Content;
    using Microsoft.Xna.Framework.Graphics;
    using Microsoft.Xna.Framework.Input;
    using Microsoft.Xna.Framework.Storage;
    using System.Threading;
    using System.Xml;
    #endregion

    using DarkWynterEngine.Globals;
    using DarkWynterEngine.ObjectLib;
    using DarkWynterEngine.Controllers;
    using DarkWynterEngine.Utilities;

    public class GameModeScreen : GameScreen
    {
        public GameModeScreen(){}

        public override void CheckMenuChoices(GameFlow gameFlow, Controller MenuController, MenuSystem menuSystem)
        {
            // GAME MODE ------------------------------------
            if (MenuController.menuInput.IsStartBackInputBlocked)
            {
                if (MenuController.menuInput.GetMenuStartBackReleased == true)
                {
                    MenuController.SetBlockStartBackInput(false);
                }
            }
            for (int i = 0; i < gameFlow.objectLibrary.humans.Count; i++)
            {
                Controller controller = gameFlow.objectLibrary.humans[i].playerController;
                controller.Update(Enums.AttackType.NONE);

                if (controller.menuInput.IsStartBackInputBlocked)
                {
                    if (controller.menuInput.GetMenuStartPressed == false && controller.menuInput.GetMenuBackPressed == false)
                    {
                        if (controller.menuInput.GetMenuStartBackReleased == true)
                        {
                            controller.SetBlockStartBackInput(false);
                        }
                    }
                }

                // If in game and user presses back
                if (controller.menuInput.GetBackPressed)
                {
                    Controller.StopRumble(gameFlow.objectLibrary);
                    MenuController.SetBlockStartBackInput(true);

                    Audio.MenuBack();
                    Audio.PlayLevelSong("Theme4");

                    Statics.gameState = Enums.EngineState.GAME_SETUP;

                    Statics.screenSaverTimer.Reset();
                    Statics.screenSaverTimer.Start();
                }
                else if (controller.menuInput.GetStartPressed && !controller.menuInput.IsStartBackInputBlocked)
                {
                    Audio.MenuStart();
                    //Pause the game
                    Statics.gameState = Enums.EngineState.GAME_PAUSE;
                    Statics.playerThatPaused = i;
                    menuSystem.pauseScreen.SetPlayerOptions(gameFlow.objectLibrary.humans[i]);

                    Statics.screenSaverTimer.Reset();
                    Statics.screenSaverTimer.Start();
                }
            }
        }
    }
}
