//---------------------------------------------------------------------------------------------------------------------------------------------------
// <copyright file="GameSetup.cs" company="DarkWynter Studios">
//     Copyright (C)2007 DarkWynter Studios.  All rights reserved.
// </copyright>
//---------------------------------------------------------------------------------------------------------------------------------------------------
// {Contact : darkwynter.com for licensing information
//---------------------------------------------------------------------------------------------------------------------------------------------------

namespace ElementalGame.GraphicUserInterface
{

    #region Using Statements
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using Microsoft.Xna.Framework;
    using Microsoft.Xna.Framework.Audio;
    using Microsoft.Xna.Framework.Content;
    using Microsoft.Xna.Framework.Graphics;
    using Microsoft.Xna.Framework.Input;
    using Microsoft.Xna.Framework.Storage;
    using System.Threading;
    using System.Xml;
    #endregion


    using DarkWynterEngine.Controllers;
    using DarkWynterEngine.Globals;


    // The Setup screen
    public class GameSetup : GameScreen
    {

        private Texture2D storyModeTexture;
        private Texture2D deathMatchTexture;
        private Texture2D survivalTexture;
        private Texture2D lastmanTexture;


        public GameSetup()
        {
            // Instantiate the GameMenu
            menu = new GameMenu("Game Setup", 100, 100);
            menu.SetDrawMenuTitle(false);

            menu.AddLabel("Game options:");
            menu.AddLabel("");
            
            menu.AddMenuOption("Game type:", "Deathmatch", "Story Mode", "Survival", "Last man standing");
            menu.AddLabel("");
            
            menu.AddValueInput("# Players:",
                Statics.GameSettings.MIN_NUMBER_OF_PLAYERS,
                Statics.GameSettings.MAX_NUMBER_OF_PLAYERS,
                Statics.GameSettings.DEFAULT_NUMBER_OF_PLAYERS,
                Statics.GameSettings.NUMBER_OF_PLAYERS_PER_TEAM
            );
            menu.AddLabel("");

            menu.AddMenuOption("Difficulty:", "Lamer", "Average Joe", "Uber Mental");
            menu.AddLabel("");
            
            menu.AddValueInput("Max kills:", 1, 10, 3, 1);
            menu.AddLabel("");
            
            menu.AddValueInput("Bots:", 0, 10, 1, 1);
            menu.AddLabel("");

            menu.AddValueInput("Bot intelligence:", 1, 10, 5, 1);
            menu.AddLabel("");

            storyModeTexture = Statics.content.Load<Texture2D>("_textures/DMatch");

        }

        public override void CheckMenuChoices(GameFlow gameFlow, Controller MenuController, MenuSystem menuSystem)
        {
            //Check for recently connecting players and update
            MenuController.SetBlockStartBackInput(false);
            int numControllersBefore = Statics.SystemSettings.TOTAL_PLAYERS;
            Controller.CheckForControllers(gameFlow.objectLibrary);

            if (Statics.SystemSettings.TOTAL_PLAYERS != numControllersBefore)
            {
                ((GameMenuValueInput)GetMenu().GetMenuElement("# Players:")).SetRange(1, Statics.SystemSettings.TOTAL_PLAYERS);
                ((GameMenuValueInput)GetMenu().GetMenuElement("# Players:")).SetValue(Statics.SystemSettings.TOTAL_PLAYERS);
            }

            base.CheckMenuChoices(gameFlow, MenuController, menuSystem);

            /*
            // Load Game ------------------------------------
            if (MenuController.menuInput.GetMenuBackPressed)
            {
                Audio.MenuBack();                       // Play a sound
                Statics.gameState = Enums.EngineState.TITLE_SCREEN;      // Switch Modes
                Statics.screenSaverTimer.Reset();
                Statics.screenSaverTimer.Start();
            }

            else if (MenuController.menuInput.GetMenuStartPressed)
            {
                // Play a sound
                Audio.MenuStart();
                Statics.screenSaverTimer.Reset();
                Statics.screenSaverTimer.Start();

                // Load Game Data
                GameSetupCommit();

                // Load Terrain Demo
                XML.LoadLevelIndex();
                //levelSetup.GetMapNameLabel().SetTitle("Map: " + XML.levelInfo[currentLevelIter].name);
                //levelSetup.GetDescriptionLabel().SetTitle(XML.levelInfo[currentLevelIter].description);

                // Switch modes
                Statics.gameState = Enums.EngineState.LOAD_LEVEL;
                menuSystem.playerSetup.PlayerSetupCommit(gameFlow.objectLibrary, menuSystem.playerSetup.playerNumber);
            }

        // If they pressed Up then get the previous element
            else if (MenuController.menuInput.GetMenuUp)
            {
                Audio.MenuUp();
                GetMenu().PreviousMenuElement();
            }
            // Or if they pressed down, get the next one
            else if (MenuController.menuInput.GetMenuDown)
            {
                Audio.MenuDown();
                GetMenu().NextMenuElement();
            }
            else if (MenuController.menuInput.GetMenuLeft)
            {
                menuSystem.UpdateMenuController(GetMenu().GetActiveMenuElement());
            }
            else if (MenuController.menuInput.GetMenuRight)
            {
                menuSystem.UpdateMenuController(GetMenu().GetActiveMenuElement());
            }
            */

            if (GetMenu().GetActiveMenuElement().GetTitle() == "Game type:")
            {
                GameMenuOption option = (GameMenuOption)GetMenu().GetActiveMenuElement();
                //need to change the settings depending on what is available here

                if (option.GetActiveOption() == "Story Mode")
                {
                    GetMenu().GetMenuElement("# Players:").Hide();
                    GetMenu().GetMenuElement("Max kills:").Hide();
                    GetMenu().GetMenuElement("Difficulty:").Show();
                    GetMenu().GetMenuElement("Bots:").Hide();
                    GetMenu().GetMenuElement("Bot intelligence:").Show();
                    SetBackground(storyModeTexture);
                }

                else if (option.GetActiveOption() == "Last man standing")
                {
                    GetMenu().GetMenuElement("# Players:").Show();
                    GetMenu().GetMenuElement("Max kills:").Hide();
                    GetMenu().GetMenuElement("Difficulty:").Hide();
                    GetMenu().GetMenuElement("Bots:").Show();
                    GetMenu().GetMenuElement("Bot intelligence:").Show();
                    SetBackground(lastmanTexture);
                }
                else if (option.GetActiveOption() == "Deathmatch")
                {
                    GetMenu().GetMenuElement("# Players:").Show();
                    GetMenu().GetMenuElement("Max kills:").Show();
                    GetMenu().GetMenuElement("Difficulty:").Hide();
                    GetMenu().GetMenuElement("Bots:").Show();
                    GetMenu().GetMenuElement("Bot intelligence:").Show();
                    SetBackground(deathMatchTexture);
                }
                else if (option.GetActiveOption() == "Survival")
                {
                    GetMenu().GetMenuElement("# Players:").Show();
                    GetMenu().GetMenuElement("Max kills:").Hide();
                    GetMenu().GetMenuElement("Difficulty:").Show();
                    GetMenu().GetMenuElement("Bots:").Hide();
                    GetMenu().GetMenuElement("Bot intelligence:").Show();
                    SetBackground(survivalTexture);
                }
            }
        }
    }
}