//---------------------------------------------------------------------------------------------------------------------------------------------------
// <copyright file="MenuSystem.cs" company="DarkWynter Studios">
//     Copyright (C)2007 DarkWynter Studios.  All rights reserved.
// </copyright>
//---------------------------------------------------------------------------------------------------------------------------------------------------
// {Contact : darkwynter.com for licensing information
//---------------------------------------------------------------------------------------------------------------------------------------------------

namespace ElementalGame.GraphicUserInterface
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using Microsoft.Xna.Framework;
    using Microsoft.Xna.Framework.Audio;
    using Microsoft.Xna.Framework.Content;
    using Microsoft.Xna.Framework.Graphics;
    using Microsoft.Xna.Framework.Input;
    using Microsoft.Xna.Framework.Storage;
    using System.Threading;
    using System.Xml;
    #endregion

    using DarkWynterEngine.Globals;
    using DarkWynterEngine.ObjectLib;
    using DarkWynterEngine.Controllers;
    using DarkWynterEngine.Utilities;

    public class MenuSystem
    {

        // CLEAN : Make a List<GameScreen>
        // Move MenuSystem ownership to GameFlow
        // Create and add GameScreens from GameFlow



        Controller MenuController;


        public TitleScreen titleScreen = new TitleScreen();                        // Idle Screen
        //public ControlsScreen controlsScreen = new ControlsScreen();
        public GameSetup gameSetup = new GameSetup();
        //public LevelSetup levelSetup = new LevelSetup();
        public PlayerSetup playerSetup = new PlayerSetup();
        public GameOverScreen gameOverScreen = new GameOverScreen();
        public LoadingScreen loadScreen = new LoadingScreen();
        public SorryScreen sorryScreen = new SorryScreen();
        public ConfirmExitScreen confirmExitScreen = new ConfirmExitScreen();
        public CreditsScreen creditsScreen = new CreditsScreen();
        public EndGameScreen endGameScreen = new EndGameScreen();
        public GameModeScreen gameModeScreen = new GameModeScreen();


        // CLEAN : SPECIAL CASE NECCESSARY??
        public PauseScreen pauseScreen = new PauseScreen();
        

        private int currentLevelIter = 0;
        public bool TERRAIN_DEMO_DONE = true;

        public MenuSystem()
        {
            // Initialize the player1 controller for menus
            MenuController = new Controller(Enums.ControllerType.PC_AND_XBOX, PlayerIndex.One);
        }

        public void UpdateMenuController(GameMenuElement activeElement)
        {
            // OptionBox
            if (activeElement.GetElementType() == Enums.MenuElementType.OPTION)
            {
                // Cast as option and change value
                GameMenuOption option = (GameMenuOption)activeElement;
                // If they pressed left, get previous option
                if (MenuController.menuInput.GetMenuLeft)
                {
                    Audio.MenuLeft();
                    option.PreviousOption();
                }
                // Or if they pressed right, get next option
                else if (MenuController.menuInput.GetMenuRight)
                {
                    Audio.MenuRight();
                    option.NextOption();
                }
            }

            // ValueSlider
            else if (activeElement.GetElementType() == Enums.MenuElementType.VALUE)
            {
                // Cast as ValueInput type and change value
                GameMenuValueInput valueInput = (GameMenuValueInput)activeElement;
                //int activeIndex = playerSetup.GetMenu().GetActiveIndex();

                // If they pressed left, decrease the value
                if (MenuController.menuInput.GetMenuLeft)
                {
                    Audio.MenuLeft();
                    valueInput.Decrement();
                }
                // Or if they pressed right, increase it
                else if (MenuController.menuInput.GetMenuRight)
                {
                    Audio.MenuRight();
                    valueInput.Increment();
                }
            }

            else if (activeElement.GetElementType() == Enums.MenuElementType.IMAGE_SCROLL)
            {
                // Cast as ValueInput type and change value
                GameMenuImageScroller scroller = (GameMenuImageScroller)activeElement;
                //int activeIndex = playerSetup.GetMenu().GetActiveIndex();

                // If they pressed left, decrease the value
                if (MenuController.menuInput.GetMenuLeft)
                {
                    Audio.MenuLeft();
                    scroller.PreviousItem();
                }
                // Or if they pressed right, increase it
                else if (MenuController.menuInput.GetMenuRight)
                {
                    Audio.MenuRight();
                    scroller.NextItem();
                }
            }
        }

        public void Update(GameFlow gameFlow)
        {
            // Update Controller Input
            MenuController.Update(Enums.AttackType.NONE);

            // Check if the FPS counter was toggled
            if (MenuController.GetFPSToggleButton())
            {
                Statics.enableFPSDisplay = !Statics.enableFPSDisplay;
            }

            if (Statics.gameState == Enums.EngineState.NOT_SUPPORTED)
            {
                //just check for quit
                if (MenuController.menuInput.GetBackPressed)
                {
                    // Exit 
                    Audio.MenuBack();           // Play a sound
                    Statics.gameState = Enums.EngineState.EXIT;
                }
            }

            // CLEAN : THESE NEED CLASSES OF THEIR OWN
            else if (Statics.gameState == Enums.EngineState.GAME_OVER)
            {
                endGameScreen.CheckMenuChoices(gameFlow, MenuController, this);
            }
            else if (Statics.gameState == Enums.EngineState.GAME_MODE)
            {
                gameModeScreen.CheckMenuChoices(gameFlow, MenuController, this);
            }
            else if (Statics.gameState == Enums.EngineState.TITLE_SCREEN)
            {
                titleScreen.CheckMenuChoices(gameFlow, MenuController, this);
            }
            else if (Statics.gameState == Enums.EngineState.GAME_SETUP)
            {
               gameSetup.CheckMenuChoices(gameFlow, MenuController, this);
            }
            else if (Statics.gameState == Enums.EngineState.GAME_PAUSE)
            {
                pauseScreen.CheckPauseScreen(gameFlow, MenuController, this);
            }
            else if (Statics.gameState == Enums.EngineState.CREDITS)
            {
                creditsScreen.CheckMenuChoices(gameFlow, MenuController, this);
            }
        }

        public void Draw(SpriteBatch spriteBatch, ScreenSaver screenSaver, GameTime gameTime)
        {
            Statics.graphics.GraphicsDevice.Clear(Color.Black);
            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Texture, SaveStateMode.None);
            Statics.graphics.GraphicsDevice.Viewport = Statics.defaultViewport;

            // Begin Screen Saver Wrapper
            if (Statics.screenSaverTimer.ElapsedMilliseconds > Statics.SystemSettings.ANTI_BURN_TIMER)
            {
                screenSaver.AntiBurnScreenSaverBegin(gameTime, spriteBatch);
            }

            // Draw the appropriate Menu Screen
            if (Statics.gameState == Enums.EngineState.NOT_SUPPORTED)
            {
                sorryScreen.Draw(spriteBatch);
            }
            else if (Statics.gameState == Enums.EngineState.CREDITS)
            {
                creditsScreen.Draw(spriteBatch);
            }
            else if (Statics.gameState == Enums.EngineState.TITLE_SCREEN)
            {
                titleScreen.Draw(spriteBatch);
            }
            else if (Statics.gameState == Enums.EngineState.GAME_SETUP)
            {
                gameSetup.Draw(spriteBatch);
            }
            else if (Statics.gameState == Enums.EngineState.LOAD_LEVEL)
            {
                Statics.GameSettings.LoadScreenLoaded = true;
                loadScreen.Draw(spriteBatch);
            }
            else if (Statics.gameState == Enums.EngineState.GAME_OVER)
            {
                gameOverScreen.Draw(spriteBatch);
            }
            else if (Statics.gameState == Enums.EngineState.GAME_PAUSE)
            {
                pauseScreen.Draw(spriteBatch);
            }

            // End Screen Saver Wrapper
            if (Statics.screenSaverTimer.ElapsedMilliseconds > Statics.SystemSettings.ANTI_BURN_TIMER)
            {
                screenSaver.AntiBurnScreenSaverEnd(gameTime, spriteBatch);
            }
            spriteBatch.End();
        }
    }
}
/*
        private void PlayerSetupCommit(ObjectLibrary objectLibrary, int playerIndex)
        {
            switch (playerIndex)
            {
                case 0:
                    {
                        // Player 1 Color Select
                        objectLibrary.humans[0].SetDiffuseRed(255);
                        objectLibrary.humans[0].SetDiffuseGreen(255);
                        objectLibrary.humans[0].SetDiffuseBlue(255);
                        objectLibrary.humans[0].SetDiffuseAlpha(255);
                        playerSetup.GetMenu().SetTitle("Player 2");
                        break;
                    }
                case 1:
                    {
                        // Player 2 Color Select
                        objectLibrary.humans[1].SetDiffuseRed(255);
                        objectLibrary.humans[1].SetDiffuseGreen(255);
                        objectLibrary.humans[1].SetDiffuseBlue(255);
                        objectLibrary.humans[1].SetDiffuseAlpha(255);
                        playerSetup.GetMenu().SetTitle("Player 3");
                        break;
                    }
                case 2:
                    {
                        // Player 3 Color Select
                        objectLibrary.humans[2].SetDiffuseRed(255);
                        objectLibrary.humans[2].SetDiffuseGreen(255);
                        objectLibrary.humans[2].SetDiffuseBlue(255);
                        objectLibrary.humans[2].SetDiffuseAlpha(255);
                        playerSetup.GetMenu().SetTitle("Player 4");
                        break;
                    }
                case 3:
                    {
                        // Player 4 Color Select
                        objectLibrary.humans[3].SetDiffuseRed(255);
                        objectLibrary.humans[3].SetDiffuseGreen(255);
                        objectLibrary.humans[3].SetDiffuseBlue(255);
                        objectLibrary.humans[3].SetDiffuseAlpha(255);
                        break;
                    }
            }

            // Player Character Select
            if (playerNumber < Statics.SystemSettings.TOTAL_PLAYERS - 1)
            {
                playerNumber++;
                Statics.currentPlayerIndex = playerNumber;
                playerSetup.GetPlayerSelection().SetCurrentIndex(0);
            }
            else
            {
                //Statics.gameState = Statics.GameState.GAME_MODE;


                playerNumber = 0;
                Statics.currentPlayerIndex = playerNumber;

                playerSetup.GetMenu().SetTitle("Player 1");
            }
        }
        private void CheckConfirmExitScreen(
            GraphicsDeviceManager graphics, ContentManager content, GameTime gameTime, 
            GameController Player1Controller, GameFlow gameFlow, DirectionInput input)
        {
            // CONFIRM EXIT SCREEN ------------------------------------
            // Exit 
            if (Player1Controller.GetMenuBackPressed())
            {
                Audio.MenuBack();           // Play a sound
                Statics.gameState = Statics.GameState.CREDITS;
            }

            // Start
            else if (Player1Controller.GetMenuStartPressed())
            {
                // Play a sound
                Audio.MenuStart();
                Statics.gameState = Statics.GameState.TITLE_SCREEN;
                Statics.screenSaverTimer.Reset();
                Statics.screenSaverTimer.Start();
            }
        }
        private void CheckLevelSetupScreen(
            GraphicsDeviceManager graphics,ContentManager content,GameTime gameTime,
            GameController Player1Controller,GameFlow gameFlow,DirectionInput input)
        {
            if (Player1Controller.IsStartBackInputBlocked())
            {
                if (Player1Controller.GetMenuStartBackReleased() == true)
                {
                    Player1Controller.SetBlockStartBackInput(false);
                }
            }

            // Load Level ------------------------------------
            if (Player1Controller.GetMenuBackPressed())
            {
                Audio.MenuBack();
                Statics.gameState = Statics.GameState.GAME_SETUP;
                Statics.loadDarkMatter = true;
            }

            else if (Player1Controller.GetMenuStartPressed())
            {
                Audio.MenuStart();
                //Set it to load mode
                Statics.gameState = Statics.GameState.LOAD_LEVEL;
                Statics.drewLoadScreen = false;
            }
            // If they pressed Up then get the previous element
            else if (input.menuUp)
            {
                Audio.MenuUp();

                currentLevelIter--;
                if (currentLevelIter < 0)
                {
                    currentLevelIter = XML.levelInfo.Count - 1;
                }
                levelSetup.GetMapNameLabel().SetTitle("Map: " + XML.levelInfo[currentLevelIter].name);
                levelSetup.GetDescriptionLabel().SetTitle(XML.levelInfo[currentLevelIter].description);
                //Highlight the right glyph
                ((GameMenuImageScroller)levelSetup.GetDisplayMenu().GetMenuElement(0)).PreviousItem();
 
                if (TERRAIN_DEMO_DONE == false)
                {
                    //must first stop thread.. 
                    //loadTerrainThread.Abort();
                    //while (loadTerrainThread.IsAlive) ;
                }
                gameFlow.LoadTerrainDemo(XML.levelInfo[currentLevelIter].filepath);//graphics, levelInfo[currentLevelIter].filepath, content);
            }
            // Or if they pressed down, get the next one
            else if (input.menuDown)
            {
                Audio.MenuDown();

                currentLevelIter++;
                if (currentLevelIter > XML.levelInfo.Count - 1)
                {
                    currentLevelIter = 0;
                }
                levelSetup.GetMapNameLabel().SetTitle("Map: " + XML.levelInfo[currentLevelIter].name);
                levelSetup.GetDescriptionLabel().SetTitle(XML.levelInfo[currentLevelIter].description);
                ((GameMenuImageScroller)levelSetup.GetDisplayMenu().GetMenuElement(0)).NextItem();
                gameFlow.LoadTerrainDemo(XML.levelInfo[currentLevelIter].filepath);//raphics, levelInfo[currentLevelIter].filepath, content);
            }
        }

        public void LevelSetupCommit(ObjectLibrary objectLibrary)
        {
        }

*/
