//---------------------------------------------------------------------------------------------------------------------------------------------------
// <copyright file="PauseScreen.cs" company="DarkWynter Studios">
//     Copyright (C)2007 DarkWynter Studios.  All rights reserved.
// </copyright>
//---------------------------------------------------------------------------------------------------------------------------------------------------
// {Contact : darkwynter.com for licensing information
//---------------------------------------------------------------------------------------------------------------------------------------------------

namespace ElementalGame.GraphicUserInterface
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using Microsoft.Xna.Framework;
    using Microsoft.Xna.Framework.Audio;
    using Microsoft.Xna.Framework.Content;
    using Microsoft.Xna.Framework.Graphics;
    using Microsoft.Xna.Framework.Input;
    using Microsoft.Xna.Framework.Storage;
    #endregion

    using DarkWynterEngine.GameObjects;
    using DarkWynterEngine.Globals;
    using DarkWynterEngine.UserInterface;
    using DarkWynterEngine.ObjectLib;
    using DarkWynterEngine.Controllers;


    // The Pause Screen
    public class PauseScreen : GameScreen
    {
        private Texture2D hintsTexture;
        private GameMenu playerPauseMenu;

        // Constructor:
        public PauseScreen()
        {
            //Instantiate the GameMenu
            menu = new GameMenu("", 0, 0);

            playerPauseMenu = new GameMenu("Paused", 150, 30);//Statics.GAME_WINDOW_WIDTH / 2, Statics.GAME_WINDOW_HEIGHT / 2);
            playerPauseMenu.SetDrawMenuTitle(false);

            playerPauseMenu.AddMenuOption("Invert rotation", "No", "Yes");
            playerPauseMenu.AddMenuOption("Rumble", "Yes", "No");
            playerPauseMenu.AddMenuOption("ShadowMap1", "Yes", "No");
            playerPauseMenu.AddMenuOption("ShadowMap2", "Yes", "No");
            playerPauseMenu.AddMenuOption("Terrain Bump Mapping", "Yes", "No");
            playerPauseMenu.AddMenuOption("Terrain Multi-Texturing", "Yes", "No");
            //playerPauseMenu.AddMenuOption("Number Of Terrain Textures", "4", "3", "2");
            playerPauseMenu.AddMenuOption("Light Sources", "2", "1");
            playerPauseMenu.AddMenuOption("Music", "On", "Off");
            playerPauseMenu.AddMenuOption("Sound Effects", "On", "Off");

            SetBackground(Statics.content.Load<Texture2D>("_textures/black"), 0.3f);
            //pauseScreen.SetHintsTexture(Statics.content.Load<Texture2D>("_textures/black"));

        }

        public GameMenu GetPlayerPauseMenu()
        {
            return playerPauseMenu;
        }

        public void SetHintsTexture(Texture2D hints)
        {
            hintsTexture = hints;
        }

        public void SetPlayerOptions(Human player)
        {
            if (player.controlsInverted)
            {
                ((GameMenuOption)playerPauseMenu.GetMenuElement("Invert rotation")).SetCurrentIndex(1);
            }
            else
            {
                ((GameMenuOption)playerPauseMenu.GetMenuElement("Invert rotation")).SetCurrentIndex(0);
            }

            if (player.rumbleEnabled)
            {
                ((GameMenuOption)playerPauseMenu.GetMenuElement("Rumble")).SetCurrentIndex(0);
            }
            else
            {
                ((GameMenuOption)playerPauseMenu.GetMenuElement("Rumble")).SetCurrentIndex(1);
            }
        }

        public void SetDefaults()
        {
            // ShadowMap1
            if (Statics.SystemSettings.enableShadowMap1)
            {
                ((GameMenuOption)playerPauseMenu.GetMenuElement("ShadowMap1")).SetCurrentIndex(0);
            }
            else
            {
                ((GameMenuOption)playerPauseMenu.GetMenuElement("ShadowMap1")).SetCurrentIndex(1);
            }

            if (Statics.SystemSettings.enableShadowMap2)
            {
                ((GameMenuOption)playerPauseMenu.GetMenuElement("ShadowMap2")).SetCurrentIndex(0);
            }
            else
            {
                ((GameMenuOption)playerPauseMenu.GetMenuElement("ShadowMap2")).SetCurrentIndex(1);
            }

            if (Statics.SystemSettings.enableTerrainBumpMapping)
            {
                ((GameMenuOption)playerPauseMenu.GetMenuElement("Terrain Bump Mapping")).SetCurrentIndex(0);
            }
            else
            {
                ((GameMenuOption)playerPauseMenu.GetMenuElement("Terrain Bump Mapping")).SetCurrentIndex(1);
            }

            if (Statics.SystemSettings.enableTerrainMultiTexturing)
            {
                ((GameMenuOption)playerPauseMenu.GetMenuElement("Terrain Multi-Texturing")).SetCurrentIndex(0);
            }
            else
            {
                ((GameMenuOption)playerPauseMenu.GetMenuElement("Terrain Multi-Texturing")).SetCurrentIndex(1);
            }

            if (Statics.SystemSettings.singleLightSource)
            {
                ((GameMenuOption)playerPauseMenu.GetMenuElement("Light Sources")).SetCurrentIndex(0);
            }
            else
            {
                ((GameMenuOption)playerPauseMenu.GetMenuElement("Light Sources")).SetCurrentIndex(1);
            }

            if (Statics.SystemSettings.music)
            {
                ((GameMenuOption)playerPauseMenu.GetMenuElement("Music")).SetCurrentIndex(0);
            }
            else
            {
                ((GameMenuOption)playerPauseMenu.GetMenuElement("Music")).SetCurrentIndex(1);
            }

            if (Statics.SystemSettings.soundFX)
            {
                ((GameMenuOption)playerPauseMenu.GetMenuElement("Sound Effects")).SetCurrentIndex(0);
            }
            else
            {
                ((GameMenuOption)playerPauseMenu.GetMenuElement("Sound Effects")).SetCurrentIndex(1);
            }
        }


        public void CheckPauseScreen(GameFlow gameFlow, Controller MenuController, MenuSystem menuSystem)
        {
            // GAME PAUSE -----------------------------------
            Controller controller = gameFlow.objectLibrary.humans[Statics.playerThatPaused].playerController;
            controller.Update(Enums.AttackType.NONE);

            if (controller.menuInput.GetStartPressed)
            {
                Audio.MenuStart();
                //Commit menu changes
                PauseMenuCommit(gameFlow.objectLibrary);

                //Resume the game
                Statics.gameState = Enums.EngineState.GAME_MODE;
                Statics.playerThatPaused = -1;

                //reset mouse cursor
                controller.JustUnPaused();
                Statics.screenSaverTimer.Reset();
                Statics.screenSaverTimer.Start();
            }
            // If they pressed Up then get the previous element
            else if (MenuController.menuInput.GetMenuUp)
            {
                Audio.MenuUp();
                GetPlayerPauseMenu().PreviousMenuElement();
            }
            // Or if they pressed down, get the next one
            else if (MenuController.menuInput.GetMenuDown)
            {
                Audio.MenuDown();
                GetPlayerPauseMenu().NextMenuElement();
            }
            else if (MenuController.menuInput.GetMenuLeft)
            {
                menuSystem.UpdateMenuController(GetPlayerPauseMenu().GetActiveMenuElement());
            }
            else if (MenuController.menuInput.GetMenuRight)
            {
                menuSystem.UpdateMenuController(GetPlayerPauseMenu().GetActiveMenuElement());
            }

            controller.AnyButtonPressed();
        }
        public void PauseMenuCommit(ObjectLibrary objectLibrary)
        {
            GameMenuOption option;      // Text Option

            // Player controller setup
            option = (GameMenuOption)GetPlayerPauseMenu().GetMenuElement("Invert rotation");
            if (option.GetActiveOption() == "Yes")
            {
                objectLibrary.humans[Statics.playerThatPaused].controlsInverted = true;
            }
            else
            {
                objectLibrary.humans[Statics.playerThatPaused].controlsInverted = false;
            }

            // Player rumble
            option = (GameMenuOption)GetPlayerPauseMenu().GetMenuElement("Rumble");
            if (option.GetActiveOption() == "Yes")
            {
                objectLibrary.humans[Statics.playerThatPaused].rumbleEnabled = true;
            }
            else
            {
                objectLibrary.humans[Statics.playerThatPaused].rumbleEnabled = false;
            }

            option = (GameMenuOption)GetPlayerPauseMenu().GetMenuElement("ShadowMap1");
            if (option.GetActiveOption() == "Yes")
            {
                Statics.SystemSettings.enableShadowMap1 = true;
            }
            else
            {
                Statics.SystemSettings.enableShadowMap1 = false;
            }

            option = (GameMenuOption)GetPlayerPauseMenu().GetMenuElement("ShadowMap2");
            if (option.GetActiveOption() == "Yes")
            {
                Statics.SystemSettings.enableShadowMap2 = true;
            }
            else
            {
                Statics.SystemSettings.enableShadowMap2 = false;
            }

            option = (GameMenuOption)GetPlayerPauseMenu().GetMenuElement("Terrain Bump Mapping");
            if (option.GetActiveOption() == "Yes")
            {
                Statics.SystemSettings.enableTerrainBumpMapping = true;
            }
            else
            {
                Statics.SystemSettings.enableTerrainBumpMapping = false;
            }

            option = (GameMenuOption)GetPlayerPauseMenu().GetMenuElement("Terrain Multi-Texturing");
            if (option.GetActiveOption() == "Yes")
            {
                Statics.SystemSettings.enableTerrainMultiTexturing = true;
            }
            else
            {
                Statics.SystemSettings.enableTerrainMultiTexturing = false;
            }

            option = (GameMenuOption)GetPlayerPauseMenu().GetMenuElement("Light Sources");
            if (option.GetActiveOption() == "1")
            {
                Statics.SystemSettings.singleLightSource = true;
                option = (GameMenuOption)GetPlayerPauseMenu().GetMenuElement("ShadowMap2");
                option.NextOption();
                //Statics.enableShadowMap2 = false;
            }
            else
            {
                Statics.SystemSettings.singleLightSource = false;
            }

            option = (GameMenuOption)GetPlayerPauseMenu().GetMenuElement("Music");
            if (option.GetActiveOption() == "On")
            {
                Statics.SystemSettings.music = true;
            }
            else
            {
                Statics.SystemSettings.music = false;
            }

            option = (GameMenuOption)GetPlayerPauseMenu().GetMenuElement("Sound Effects");
            if (option.GetActiveOption() == "On")
            {
                Statics.SystemSettings.soundFX = true;
            }
            else
            {
                Statics.SystemSettings.soundFX = false;
            }
        }

        public void Draw(SpriteBatch spriteBatch, GraphicsDevice graphicsDevice)
        {
            graphicsDevice.Viewport = Statics.defaultViewport;
            base.Draw(spriteBatch);
            // Now draw the menu for THIS player
            if (Statics.playerThatPaused != -1)
            {
                //Figure out how many players there are and if we need to shrink the font
                if (Statics.SystemSettings.TOTAL_PLAYERS > 2)
                {
                    //use smaller font
                    playerPauseMenu.SetFont(FontWriter.ComicSansSmall);
                }
                else
                {
                    //use bigger font
                    playerPauseMenu.SetFont(FontWriter.ComicSans);
                }
                spriteBatch.End();
                graphicsDevice.Viewport = Statics.viewportList[Statics.playerThatPaused].viewport;
                spriteBatch.Begin(SpriteBlendMode.AlphaBlend);
                //draw hints background
                if (hintsTexture != null)
                {
                    spriteBatch.Draw(hintsTexture, new Rectangle(0,0,graphicsDevice.Viewport.Width, graphicsDevice.Viewport.Height), Color.White);
                }
                playerPauseMenu.Draw(spriteBatch);
                spriteBatch.End();
                //restart spritebatch for later
                spriteBatch.Begin(SpriteBlendMode.AlphaBlend);
            }
        }
    }
}