//---------------------------------------------------------------------------------------------------------------------------------------------------
// <copyright file="SkySphere.cs" company="DarkWynter Studios">
//     Copyright (C)2007 DarkWynter Studios.  All rights reserved.
// </copyright>
//---------------------------------------------------------------------------------------------------------------------------------------------------
// {Contact : darkwynter.com for licensing information
//---------------------------------------------------------------------------------------------------------------------------------------------------

namespace ElementalGame.GameObjects
{
    #region Using Statements
    using System;
    using System.Collections.Generic;
    using Microsoft.Xna.Framework;
    using Microsoft.Xna.Framework.Audio;
    using Microsoft.Xna.Framework.Content;
    using Microsoft.Xna.Framework.Graphics;
    using Microsoft.Xna.Framework.Input;
    using Microsoft.Xna.Framework.Storage;
    using System.Xml;
    #endregion

    using DarkWynterEngine.Globals;
    using DarkWynterEngine.ObjectLib;
    using DarkWynterEngine.GameObjects;



    public class SkySphere : GameObject
    {
        private string skyTexture;
        private string skyTextureBump;
        private float rotationScale = 10.0f;

        private float[] materialDiffuse = { 0.7f, 0.7f, 0.7f, 1.0f };

        // Specular factor
        private float[] materialSpecular = { 0.0f, 0.0f, 0.0f, 1.0f };

        /* GameObject Interface Methods
         *   
         */
        public SkySphere()
        {
            mass.gameObjectPointer = this;
            mass.objectType = Enums.ObjectType.SKYSPHERE;
        }

        public override bool Load(XmlNode node, ObjectLibrary objectLibrary)
        {
            
            //base.Load(node,  objectLibrary);

            try
            {
                skyTexture = node.Attributes["texture"].Value;
                skyTextureBump = node.Attributes["bumpTexture"].Value;
            }
            catch { System.Diagnostics.Debug.WriteLine("Error reading sky texture filename"); return false; }

            textureList.Clear();
            textureList.Add(Statics.content.Load<Texture2D>(skyTexture));
            textureList.Add(Statics.content.Load<Texture2D>(skyTextureBump));

            model = Statics.content.Load<Model>("_models/skySphere");
            effect = Statics.content.Load<Effect>("_shaders/ElementalGPU");

            // Apply our own advanced Effect to the Model
            foreach (ModelMesh mesh in model.Meshes)
            {
                for (int i = 0; i < mesh.MeshParts.Count; i++)
                {
                    // Add effect to mesh
                    mesh.MeshParts[i].Effect = effect;
                }
            }

            mass.SetPosition(new Vector3(Statics.collisionMapSize / 2, 500, Statics.collisionMapSize / 2), new Vector3(Statics.collisionMapSize / 2, 500, Statics.collisionMapSize / 2 + 1));
            mass.scale = new Vector3(Statics.vertexMapSize * 8);
            //skySphere.mass.scale = Statics.terrainScaleFactor * 32;

            return true;
        }

        public override void Update(ref ObjectLibrary objectLibrary)
        {
            Random rand = new Random();
            //// Increment Rotation With dt
            mass.Rotate(0.00f, 0.01f);
        }

        public override void Draw(ModelManager modelManager, string technique)
        {
            // HACK ==========================================
            return;


            matrix = Matrix.CreateScale(mass.scale) *
                     Matrix.CreateFromQuaternion(mass.currentRotation) *
                     Matrix.CreateTranslation(mass.currentPosition);

            ShaderParameters.World.SetValue(matrix);

            // Lighting
            ShaderParameters.materialDiffuse.SetValue(materialDiffuse);
            ShaderParameters.materialSpecular.SetValue(materialSpecular);
            ShaderParameters.modelTexture1.SetValue(textureList[0]);
            ShaderParameters.bumpTexture1.SetValue(textureList[1]);

            // Draw the model
            base.DrawTriStrips(model, technique);
        }
    }
}