//===================================================================================================
// Uniform Variables
//===================================================================================================
float dt;

float4x4 ViewProj;
float4x4 World;
float4x4 WorldInverseTranspose;

float gpuObjectScale;
float gravity = 0.01f;

float scaleMinY;
float scaleRangeY;

float cpScaleMinY;
float cpScaleRangeY;

float2 playerPosition;

float invSpatMapHeight;
float invSpatMapWidth;

int IndexedPass;

//===================================================================================================
// Object Properties
//===================================================================================================

//- LocationMap --
texture LocationMap; 
sampler LocationMapSampler = sampler_state 
{
	texture		= <LocationMap>; 
	AddressU	= CLAMP;	
	AddressV	= CLAMP;	
	AddressW	= CLAMP; 
	MIPFILTER	= POINT;	
	MINFILTER	= POINT;	
	MAGFILTER	= POINT;
};

//- TerrainHeightMap --
texture TerrainHeightMap;
sampler TerrainHeightMapSampler = sampler_state 
{
	texture		= <TerrainHeightMap>; 
	AddressU	= CLAMP;	
	AddressV	= CLAMP;	
	AddressW	= CLAMP; 
	MIPFILTER	= POINT;	
	MINFILTER	= POINT;	
	MAGFILTER	= POINT;
};

//- TerrainNormalMap --
texture TerrainNormalMap;
sampler TerrainNormalMapSampler = sampler_state 
{
	texture		= <TerrainNormalMap>; 
	AddressU	= CLAMP;	
	AddressV	= CLAMP;	
	AddressW	= CLAMP; 
	MIPFILTER	= POINT;	
	MINFILTER	= POINT;	
	MAGFILTER	= POINT;
};

//- Position --
texture CurrentPosition; 
sampler CurrentPositionSampler = sampler_state 
{
	texture		= <CurrentPosition>; 
	AddressU	= CLAMP;	
	AddressV	= CLAMP;	
	AddressW	= CLAMP; 
	MIPFILTER	= POINT;	
	MINFILTER	= POINT;	
	MAGFILTER	= POINT;
};

//- Rotation --
texture Rotation; 
sampler RotationSampler = sampler_state 
{
	texture		= <Rotation>; 
	AddressU	= CLAMP;	
	AddressV	= CLAMP;	
	AddressW	= CLAMP; 
	MIPFILTER	= POINT;	
	MINFILTER	= POINT;	
	MAGFILTER	= POINT;
};

//- Scale --
texture Scale; 
sampler ScaleSampler = sampler_state 
{
	texture		= <Scale>; 
	AddressU	= CLAMP;	
	AddressV	= CLAMP;	
	AddressW	= CLAMP; 
	MIPFILTER	= POINT;	
	MINFILTER	= POINT;	
	MAGFILTER	= POINT;
};

//- Mass --
texture Mass; 
sampler MassSampler = sampler_state 
{
	texture		= <Mass>;
	AddressU	= CLAMP;	
	AddressV	= CLAMP;	
	AddressW	= CLAMP; 
	MIPFILTER	= POINT;	
	MINFILTER	= POINT;	
	MAGFILTER	= POINT;
};

//- Velocity --
texture Velocity; 
sampler VelocitySampler = sampler_state 
{
	texture		= <Velocity>;
	AddressU	= CLAMP;	
	AddressV	= CLAMP;	
	AddressW	= CLAMP; 
	MIPFILTER	= POINT;	
	MINFILTER	= POINT;	
	MAGFILTER	= POINT;
};

//- Acceleration --
texture Acceleration; 
sampler AccelerationSampler = sampler_state 
{
	texture		= <Acceleration>; 
	AddressU	= CLAMP;	
	AddressV	= CLAMP;	
	AddressW	= CLAMP; 
	MIPFILTER	= POINT;	
	MINFILTER	= POINT;	
	MAGFILTER	= POINT;
};

//- LastPosition --
texture LastPosition; 
sampler LastPositionSampler = sampler_state 
{
	texture		= <LastPosition>;
	AddressU	= CLAMP;	
	AddressV	= CLAMP;	
	AddressW	= CLAMP; 
	MIPFILTER	= POINT;	
	MINFILTER	= POINT;	
	MAGFILTER	= POINT;
};

//- TotalForce --
texture TotalForce; 
sampler TotalForceSampler = sampler_state 
{
	texture		= <TotalForce>; 
	AddressU	= CLAMP;	
	AddressV	= CLAMP;	
	AddressW	= CLAMP; 
	MIPFILTER	= POINT;	
	MINFILTER	= POINT;	
	MAGFILTER	= POINT;
};

//- WindMap --
texture WindMap; 
sampler WindMapSampler = sampler_state 
{
	texture		= <WindMap>; 
	AddressU	= CLAMP;	
	AddressV	= CLAMP;	
	AddressW	= CLAMP; 
	MIPFILTER	= POINT;	
	MINFILTER	= POINT;	
	MAGFILTER	= POINT;
};

//- Radius --
texture Radius; 
sampler RadiusSampler = sampler_state 
{
	texture		= <Radius>; 
	AddressU	= CLAMP;	
	AddressV	= CLAMP;	
	AddressW	= CLAMP; 
	MIPFILTER	= POINT;	
	MINFILTER	= POINT;	
	MAGFILTER	= POINT;
};

//- Dummy --
texture Dummy; 
sampler DummySampler = sampler_state 
{
	texture		= <Dummy>; 
	AddressU	= CLAMP;	
	AddressV	= CLAMP;	
	AddressW	= CLAMP; 
	MIPFILTER	= POINT;	
	MINFILTER	= POINT;	
	MAGFILTER	= POINT;
};


//===================================================================================================
// Uniform, Varying, and FrameBuffer "Stream" Structures - 
// Used to Communicate between CPU, VertexProcessor, FragmentProcessor, and FrameBuffer
//===================================================================================================
struct VertexInput 
{
    float4	position				: POSITION0;		// Position
	float4  fog						: FOG;

    float4	color					: COLOR0;			// Color
    float4	texCoord				: TEXCOORD0;		// Texture coordinates
    float3	normal					: NORMAL0;			// Normal vector
    float4	tangent					: TANGENT0;			// Tangent
    float4	binormal				: BINORMAL0;		// Binormal

	half4	indices					: BLENDINDICES0;
	float4	weights					: BLENDWEIGHT0;
	float	pointSize				: PSIZE0;			// Point size
	float	tessFactor				: TESSFACTOR0; 		// Tessellation factor	
};

struct VertexOutput
{
	// These values are used directly by the rasterizer
	float4	position				: POSITION; 		// Position
	float2	fog						: FOG; 			// Point size

	// These values are interpolated and sent to FragIn
	float4	color					: COLOR0; 			// Color (example: COLOR0)
	float4	texCoord				: TEXCOORD0; 		// Texture coordinates (example: TEXCOORD0)
    float3	normal 					: TEXCOORD1;		// Texture coordinates 
    float3	worldPos  				: TEXCOORD2;		// Texture coordinates 
    float4	shadowMapSamplingPos1	: TEXCOORD3;		// Texture coordinates 	
	float4	shadowMapSamplingPos2	: TEXCOORD4;		// Texture coordinates 	
	float2	realDepth				: TEXCOORD5;
	float3	tangent					: TEXCOORD6;
	float3	binormal				: TEXCOORD7;
};

struct FragmentInput
{
	float4	color					: COLOR0; 			// Color (example: COLOR0)
	float2	fog						: FOG;
    float4	texCoord				: TEXCOORD0;		// Texture coordinates 
    float3	normal					: TEXCOORD1;		// Texture coordinates 
    float3	worldPos				: TEXCOORD2;		// Texture coordinates 
	float4	shadowMapSamplingPos1	: TEXCOORD3;
	float4	shadowMapSamplingPos2	: TEXCOORD4;		// Texture coordinates 	
	float2	realDepth				: TEXCOORD5;
	float3	tangent					: TEXCOORD6;
	float3	binormal				: TEXCOORD7;
};

struct FragmentOutput
{
	float4	color				: COLOR0; 			// Color for render target n
};
 

//===================================================================================================
// Utility Functions
//===================================================================================================

// Gets Indexed Coordinates from an objects Spatial Position Data
float4 GetIndexedPosition(float2 spatialPosition)
{
    return tex2D(LocationMapSampler, spatialPosition);   
}

// Gets Spatial Coordinates from an objects Indexed Position Data
float4 GetSpatialPosition(float2 indexedPosition)
{
    return tex2D(CurrentPositionSampler, indexedPosition);
}

float GetUnscaledHeight(float2 texCoord)
{
	float2 hackTexCoords;
	
	hackTexCoords.xy = texCoord.yx;
	
	float height = tex2D(TerrainHeightMapSampler, hackTexCoords).g * 3.0f ;
	height += tex2D(TerrainHeightMapSampler, float2(hackTexCoords.x-3, hackTexCoords.y+3)).g;
	height += tex2D(TerrainHeightMapSampler, float2(hackTexCoords.x+3, hackTexCoords.y+3)).g;
	height += tex2D(TerrainHeightMapSampler, float2(hackTexCoords.x-3, hackTexCoords.y-3)).g;
	height += tex2D(TerrainHeightMapSampler, float2(hackTexCoords.x+3, hackTexCoords.y-3)).g;
	height = (height) / 7.0f;
	
	height = (height * scaleRangeY) + scaleMinY;
	height = (height - cpScaleMinY) / cpScaleRangeY;
		
	return height;
}

//===================================================================================================
// Indexed Functions
//===================================================================================================

void FS_Dummy(inout FragmentInput IN, inout FragmentOutput OUT)
{
	//--- HACK : READ BELOW ----
	// This function exists b/c the our multipass sequence must be "null-terminated"...
	// Because of something with xna, or dx, or my code.. the last Render To Texture in a sequence will sometimes come out black.   
	// So we make a extra trivial pass so the graphics card has a chance to finalize the previous texture.
	
	OUT.color = tex2D(DummySampler, IN.texCoord);
}

//--- Equation: acceleration = totalForce / mass;
void FS_Acceleration(inout FragmentInput IN, inout FragmentOutput OUT)
{
    float4 tf = tex2D(TotalForceSampler, IN.texCoord);
    float4 m = tex2D(MassSampler, IN.texCoord);

	OUT.color.r = tf.r/m.r; 
	OUT.color.g = tf.g/m.r; 
	OUT.color.b = tf.b/m.r; 
	OUT.color.a = tf.a/m.r; 
}


//--- Equation: velocity += acceleration * dt;
void FS_Velocity(inout FragmentInput IN, inout FragmentOutput OUT)
{
    float4 a = tex2D(AccelerationSampler, IN.texCoord);
    float4 v0 = tex2D(VelocitySampler, IN.texCoord);

	// v = v0 + at
    OUT.color = v0+ a * dt; 
}


//--- Equation: angular momentum
void FS_Rotation(inout FragmentInput IN, inout FragmentOutput OUT)
{
    OUT.color = tex2D(RotationSampler, IN.texCoord);
}

//--- Equation: currentPosition = lastPosition + velocity * dt + (0.5f * acceleration * dt * dt);
void FS_CurrentPosition(inout FragmentInput IN, inout FragmentOutput OUT)
{
	// Get Parameters    
    float4 lp = tex2D(CurrentPositionSampler, IN.texCoord);
    float4 v = tex2D(VelocitySampler, IN.texCoord);
    float4 a = tex2D(AccelerationSampler, IN.texCoord); 
    
	// Calc new position using standard physics
	OUT.color = lp + v*dt + (0.5f * a * dt*dt);
	OUT.color.a = 1.0f;

	// Bounding for TerrainMap Positions (ENTIRE BOARD)
	float xmin = 0.0f;
	float xmax = 1.00f;
	
	float ymin = -0.1f;
	float ymax = 0.5f;
	
	float zmin = 0.0f;
	float zmax = 1.0f;
	
	// Adjust to player's position
	//xmin = playerPosition.x - .05;
	//xmax = playerPosition.x + .05;
	//zmin = playerPosition.y - .05;
	//zmax = playerPosition.y + .05;

	// ------------ Start Object Recycler --------------	
	// Axis Mins to Maxs
	if(OUT.color.x < xmin){ OUT.color.x = xmax; }
	// -- REPLACED BY TERRAIN COLLISION -- 
	// if(OUT.color.y < ymin){ OUT.color.y = ymax; }
	if(OUT.color.z < zmin){ OUT.color.z = zmax; }
	
	// Axis Maxs to Mins
	if(OUT.color.x > xmax){ OUT.color.x = xmin; }
	if(OUT.color.y > ymax){ OUT.color.y = ymin; }
	if(OUT.color.z > zmax){ OUT.color.z = zmin; }
	// ------------ End Object Recycler --------------
	
	
	// ------------ Start Object Reset --------------	
    // Object distance below terrain (positive if below terrain).
    float objectResetHeight = GetUnscaledHeight(OUT.color.xz) - OUT.color.g; 
    
	//-- Reset Object Position To Terrain Height
	if( objectResetHeight > 0.0f )
	{
		// Add difference to object's height value
		OUT.color.g += objectResetHeight; 
	}      	
	// ------------ End Object Reset --------------		
}

//--- Trivial vertex shader for Indexed Fragment Shaders
void VS_Indexed(inout VertexInput IN, inout VertexOutput OUT)
{
    // Copy OUT to IN
    OUT.position = mul(IN.position, ViewProj);
    OUT.texCoord = IN.texCoord;
}



//===================================================================================================
// Collision Detection
//===================================================================================================
float4 GetForce(float2 spatialPosition)
{	
    // Get Indexed Position of Object from SpatialPosition
    float4 ip = tex2D(LocationMapSampler, spatialPosition);   

	// If object exists return it's force
	if(ip.a > 0.0f){
	    float4 v = tex2D(VelocitySampler, ip.xy);   
		float4 m = tex2D(MassSampler, ip.xy);   

		// Low-pass filter * velocity * mass (only stored in r channel)
	    return v * (float4)m.r;
	}
	else{
		return (float4)0.0f;
	}
}

void FS_TotalForce(inout FragmentInput IN, inout FragmentOutput OUT)
{   
    // Indexed CurrentPosition holds the object's Spatial Location
    float4 spatialPosition = tex2D(CurrentPositionSampler, IN.texCoord);
	
//	if(false)
	if(spatialPosition.a = 1.0f)
	{
		// Get any surrounding objects, and apply resultant force    
		float4 sumForces = (float4)0.0f;  
		      
		sumForces += GetForce( float2(1 - spatialPosition.x - invSpatMapWidth	, spatialPosition.z - invSpatMapHeight) );
		sumForces += GetForce( float2(1 - spatialPosition.x - invSpatMapWidth	, spatialPosition.z					  ) );
		sumForces += GetForce( float2(1 - spatialPosition.x - invSpatMapWidth	, spatialPosition.z + invSpatMapHeight) );
		sumForces += GetForce( float2(1 - spatialPosition.x						, spatialPosition.z - invSpatMapHeight) );
		sumForces += GetForce( float2(1 - spatialPosition.x						, spatialPosition.z + invSpatMapHeight) );
		sumForces += GetForce( float2(1 - spatialPosition.x + invSpatMapWidth	, spatialPosition.z - invSpatMapHeight) );
		sumForces += GetForce( float2(1 - spatialPosition.x + invSpatMapWidth	, spatialPosition.z					  ) );
		sumForces += GetForce( float2(1 - spatialPosition.x + invSpatMapWidth	, spatialPosition.z + invSpatMapHeight) );
		OUT.color = sumForces / 8.0f;
	}

	// Wind Force (Scale to match lastForce value)
	float4 windForce = tex2D(WindMapSampler, spatialPosition.xz);	// Get wind force from map
	windForce = (windForce - .5f) / 100000.0f;						
	
	// Get Last TotalForce (removing gravity component)
	float4 lastForce = tex2D(TotalForceSampler, IN.texCoord);
	lastForce.g += .00065f;		
	    
	// Resultant wind force = windFriction - windForce;
	OUT.color += (.25f * lastForce- windForce);// ;
	    
	// Add Gravity
	OUT.color.g -= .00065f;
    
	// ------------ Start Object Reflex --------------	
    // Object distance below terrain (positive if below terrain).
    float objectReflexHeight = GetUnscaledHeight(OUT.color.xz) - OUT.color.g; 
    
	//-- Reset Object Position To Terrain Height
	if( objectReflexHeight > 000.0f )
	{
		// Add difference to object's height value
		//OUT.color.g += .0012f;
		//OUT.color.r = -OUT.color.r;
		//OUT.color.b = -OUT.color.b; 
	}      	
	// ------------ End Object Reflex --------------		     

	OUT.color.a = 1.0f;
}

void FS_AttackForce(inout FragmentInput IN, inout FragmentOutput OUT)
{   

}

// Indexed Mains
//--------------------------------------------------------------------------------------------------
//--- Indexed Vertex Shaders
VertexOutput IndexedVertexMain(VertexInput VS_IN)
{
	VertexOutput VS_OUT = (VertexOutput)0;
	VS_Indexed(VS_IN, VS_OUT);
	return VS_OUT;
}

//--- Indexed Fragment Shaders
FragmentOutput TotalForceFragmentMain(FragmentInput FS_IN)
{
    FragmentOutput FS_OUT = (FragmentOutput)0;
    FS_TotalForce(FS_IN, FS_OUT);        
	return FS_OUT;
}
FragmentOutput AccelerationFragmentMain(FragmentInput FS_IN)
{
    FragmentOutput FS_OUT = (FragmentOutput)0;
    FS_Acceleration(FS_IN, FS_OUT);            
	return FS_OUT;
}
FragmentOutput VelocityFragmentMain(FragmentInput FS_IN)
{
    FragmentOutput FS_OUT = (FragmentOutput)0;
    FS_Velocity(FS_IN, FS_OUT);        
	return FS_OUT;
}
FragmentOutput RotationFragmentMain(FragmentInput FS_IN)
{
    FragmentOutput FS_OUT = (FragmentOutput)0;
    FS_Rotation(FS_IN, FS_OUT);            
	return FS_OUT;
}
FragmentOutput CurrentPositionFragmentMain(FragmentInput FS_IN)
{
    FragmentOutput FS_OUT = (FragmentOutput)0;
    FS_CurrentPosition(FS_IN, FS_OUT);            
	return FS_OUT;
}
FragmentOutput DummyFragmentMain(FragmentInput FS_IN)
{
    FragmentOutput FS_OUT = (FragmentOutput)0;
    FS_Dummy(FS_IN, FS_OUT);            
	return FS_OUT;
}

// Indexed Techniques
//--------------------------------------------------------------------------------------------------
technique TotalForce
{
    pass Pass0
    {
        VertexShader = compile vs_3_0 IndexedVertexMain();
        PixelShader = compile ps_3_0 TotalForceFragmentMain();        
        AlphaBlendEnable = true; SrcBlend = SrcAlpha; DestBlend = InvSrcAlpha;
		FillMode = Solid;
    }
}
technique Acceleration
{
    pass Pass0
    {
        VertexShader = compile vs_3_0 IndexedVertexMain();
        PixelShader = compile ps_3_0 AccelerationFragmentMain();        
        AlphaBlendEnable = true; SrcBlend = SrcAlpha; DestBlend = InvSrcAlpha;
		FillMode = Solid;
    }
}
technique Velocity
{
    pass Pass0
    {
        VertexShader = compile vs_3_0 IndexedVertexMain();
        PixelShader = compile ps_3_0 VelocityFragmentMain();        
        AlphaBlendEnable = true; SrcBlend = SrcAlpha; DestBlend = InvSrcAlpha;
		FillMode = Solid;
    }
}
technique Rotation
{
    pass Pass0
    {
        VertexShader = compile vs_3_0 IndexedVertexMain();
        PixelShader = compile ps_3_0 RotationFragmentMain();        
        AlphaBlendEnable = true; SrcBlend = SrcAlpha; DestBlend = InvSrcAlpha;
		FillMode = Solid;
    }
}
technique CurrentPosition
{
    pass Pass0
    {
        VertexShader = compile vs_3_0 IndexedVertexMain();
        PixelShader = compile ps_3_0 CurrentPositionFragmentMain();        
        AlphaBlendEnable = true; SrcBlend = SrcAlpha; DestBlend = InvSrcAlpha;
		FillMode = Solid;
    }
}
technique Dummy
{
    pass Pass0
    {
        VertexShader = compile vs_3_0 IndexedVertexMain();
        PixelShader = compile ps_3_0 DummyFragmentMain();        
        AlphaBlendEnable = true; SrcBlend = SrcAlpha; DestBlend = InvSrcAlpha;
		FillMode = Solid;
    }
}



//==================================================================================================
// Spatial Function
// Using a Quad of Verts mapped to the CurrentPosition indexed map,
// relocate each vert to it's spatial position and render to LocationMap
//==================================================================================================
void FS_Spatial(inout FragmentInput IN, inout FragmentOutput OUT)
{    
    OUT.color = IN.color;
}

void VS_Spatial(inout VertexInput IN, inout VertexOutput OUT)
{	    
    // Set Vertex Position to the location stored in Position Indexed Map
    OUT.position = tex2Dlod(CurrentPositionSampler, float4(IN.texCoord.x, IN.texCoord.y, 0, 0));    // IN.position;
    OUT.position.g = OUT.position.b;
    OUT.position.b = 0.0f;
    
    OUT.position = mul(OUT.position, ViewProj);

    // Pack data into pixel values
    OUT.color.rg = IN.texCoord.xy;          // Pointer to Indexed Object
    OUT.color.b = 0.0f;						// Empty
    OUT.color.a = 1.0f;                     // isObject Flag - marks object as active

	//HACK FOR VISIBILITY
	//OUT.color.r = 1.0f;

    // Pass texCoords along
    OUT.texCoord = IN.texCoord;    
}

// Spatial Mains
//--------------------------------------------------------------------------------------------------
VertexOutput SpatialVertexMain(VertexInput VS_IN)
{
	VertexOutput VS_OUT = (VertexOutput)0;
	VS_Spatial(VS_IN, VS_OUT);
	return VS_OUT;
}
FragmentOutput SpatialFragmentMain(FragmentInput FS_IN)
{
    FragmentOutput FS_OUT = (FragmentOutput)0;
	FS_Spatial(FS_IN, FS_OUT);
	return FS_OUT;
}


// SpatialMapping Techniques
//--------------------------------------------------------------------------------------------------
technique SpatialMain
{
    pass Pass0
    {
        VertexShader = compile vs_3_0 SpatialVertexMain();
        PixelShader = compile ps_3_0 SpatialFragmentMain();
        
        AlphaBlendEnable = true; SrcBlend = SrcAlpha; DestBlend = InvSrcAlpha;
		FillMode = Solid;
    }
}

